@@ -50,8 +50,6 @@
 #include "core/manager.h"
 #include "core/view_manager.h"
 #include "rpc/command_scheduler.h"
-#include "rpc/command_slot.h"
-#include "rpc/command_variable.h"
 #include "rpc/parse.h"
 #include "rpc/parse_commands.h"
 
@@ -59,53 +57,21 @@
 #include "control.h"
 #include "command_helpers.h"
 
-torrent::Object
-apply_on_state_change(const char* name, const torrent::Object& rawArgs) {
-  const torrent::Object::list_type& args = rawArgs.as_list();
-
-  if (args.size() == 0 || args.size() > 2)
-    throw torrent::input_error("Wrong number of arguments.");
-
-  const std::string& rawKey = args.front().as_string();
-
-  if (rawKey.empty())
-    throw torrent::input_error("Empty key.");
-
-  // If the key starts with '_' then it's supposed to be literal, with
-  // the initial '_' removed. This allows us to get proper ordering
-  // for internal rtorrent tasks.
-  std::string key = rawKey[0] != '_' ? ("1_state_" + rawKey) : rawKey.substr(1);
-
-  if (args.size() == 1)
-    rpc::commands.call("system.method.set_key", rpc::make_target(), rpc::create_object_list(name, key));
-  else
-    rpc::commands.call("system.method.set_key", rpc::make_target(), rpc::create_object_list(name, key, args.back()));
-
-  // Deprecated notice, remove this function in the next minor
-  // version.
-  static bool notify = true;
-
-  if (notify) {
-    control->core()->push_log("Deprecated on_* commands, use 'system.method.set_key = event.download.{inserted, erased, ...}, <key>, <command>' instead.");
-    notify = false;
-  }
-
-  return torrent::Object();
-}
+#include "thread_worker.h"
 
 torrent::Object
 apply_on_ratio(const torrent::Object& rawArgs) {
   const std::string& groupName = rawArgs.as_string();
 
   char buffer[32 + groupName.size()];
-  sprintf(buffer, "group.%s.view", groupName.c_str());
+  sprintf(buffer, "group2.%s.view", groupName.c_str());
 
   core::ViewManager::iterator viewItr = control->view_manager()->find(rpc::commands.call(buffer, rpc::make_target()).as_string());
 
   if (viewItr == control->view_manager()->end())
     throw torrent::input_error("Could not find view.");
 
-  char* bufferStart = buffer + sprintf(buffer, "group.%s.ratio.", groupName.c_str());
+  char* bufferStart = buffer + sprintf(buffer, "group2.%s.ratio.", groupName.c_str());
 
   // first argument:  minimum ratio to reach
   // second argument: minimum upload amount to reach [optional]
@@ -120,13 +86,13 @@ apply_on_ratio(const torrent::Object& rawArgs) {
   std::vector<core::Download*> downloads;
 
   for  (core::View::iterator itr = (*viewItr)->begin_visible(), last = (*viewItr)->end_visible(); itr != last; itr++) {
-    if (!(*itr)->is_seeding() || rpc::call_command_value("d.get_ignore_commands", rpc::make_target(*itr)) != 0)
+    if (!(*itr)->is_seeding() || rpc::call_command_value("d.ignore_commands", rpc::make_target(*itr)) != 0)
       continue;
 
     //    rpc::parse_command_single(rpc::make_target(*itr), "print={Checked ratio of download.}");
 
     int64_t totalDone   = (*itr)->download()->bytes_done();
-    int64_t totalUpload = (*itr)->download()->up_rate()->total();
+    int64_t totalUpload = (*itr)->info()->up_rate()->total();
 
     if (!(totalUpload >= minUpload && totalUpload * 100 >= totalDone * minRatio) &&
         !(maxRatio > 0 && totalUpload * 100 > totalDone * maxRatio))
@@ -135,7 +101,7 @@ apply_on_ratio(const torrent::Object& rawArgs) {
     downloads.push_back(*itr);
   }
 
-  std::strcpy(bufferStart, "command");
+  sprintf(buffer, "group.%s.ratio.command", groupName.c_str());
 
   for (std::vector<core::Download*>::iterator itr = downloads.begin(), last = downloads.end(); itr != last; itr++) {
     //    rpc::commands.call("print", rpc::make_target(*itr), "Calling ratio command.");
@@ -148,11 +114,11 @@ apply_on_ratio(const torrent::Object& rawArgs) {
 torrent::Object
 apply_start_tied() {
   for (core::DownloadList::iterator itr = control->core()->download_list()->begin(); itr != control->core()->download_list()->end(); ++itr) {
-    if (rpc::call_command_value("d.get_state", rpc::make_target(*itr)) == 1)
+    if (rpc::call_command_value("d.state", rpc::make_target(*itr)) == 1)
       continue;
 
     rak::file_stat fs;
-    const std::string& tiedToFile = rpc::call_command_string("d.get_tied_to_file", rpc::make_target(*itr));
+    const std::string& tiedToFile = rpc::call_command_string("d.tied_to_file", rpc::make_target(*itr));
 
     if (!tiedToFile.empty() && fs.update(rak::path_expand(tiedToFile)))
       rpc::parse_command_single(rpc::make_target(*itr), "d.try_start=");
@@ -164,11 +130,11 @@ apply_start_tied() {
 torrent::Object
 apply_stop_untied() {
   for (core::DownloadList::iterator itr = control->core()->download_list()->begin(); itr != control->core()->download_list()->end(); ++itr) {
-    if (rpc::call_command_value("d.get_state", rpc::make_target(*itr)) == 0)
+    if (rpc::call_command_value("d.state", rpc::make_target(*itr)) == 0)
       continue;
 
     rak::file_stat fs;
-    const std::string& tiedToFile = rpc::call_command_string("d.get_tied_to_file", rpc::make_target(*itr));
+    const std::string& tiedToFile = rpc::call_command_string("d.tied_to_file", rpc::make_target(*itr));
 
     if (!tiedToFile.empty() && !fs.update(rak::path_expand(tiedToFile)))
       rpc::parse_command_single(rpc::make_target(*itr), "d.try_stop=");
@@ -181,9 +147,9 @@ torrent::Object
 apply_close_untied() {
   for (core::DownloadList::iterator itr = control->core()->download_list()->begin(); itr != control->core()->download_list()->end(); ++itr) {
     rak::file_stat fs;
-    const std::string& tiedToFile = rpc::call_command_string("d.get_tied_to_file", rpc::make_target(*itr));
+    const std::string& tiedToFile = rpc::call_command_string("d.tied_to_file", rpc::make_target(*itr));
 
-    if (rpc::call_command_value("d.get_ignore_commands", rpc::make_target(*itr)) == 0 && !tiedToFile.empty() && !fs.update(rak::path_expand(tiedToFile)))
+    if (rpc::call_command_value("d.ignore_commands", rpc::make_target(*itr)) == 0 && !tiedToFile.empty() && !fs.update(rak::path_expand(tiedToFile)))
       rpc::parse_command_single(rpc::make_target(*itr), "d.try_close=");
   }
 
@@ -194,11 +160,11 @@ torrent::Object
 apply_remove_untied() {
   for (core::DownloadList::iterator itr = control->core()->download_list()->begin(); itr != control->core()->download_list()->end(); ) {
     rak::file_stat fs;
-    const std::string& tiedToFile = rpc::call_command_string("d.get_tied_to_file", rpc::make_target(*itr));
+    const std::string& tiedToFile = rpc::call_command_string("d.tied_to_file", rpc::make_target(*itr));
 
     if (!tiedToFile.empty() && !fs.update(rak::path_expand(tiedToFile))) {
       // Need to clear tied_to_file so it doesn't try to delete it.
-      rpc::call_command("d.set_tied_to_file", std::string(), rpc::make_target(*itr));
+      rpc::call_command("d.tied_to_file.set", std::string(), rpc::make_target(*itr));
 
       itr = control->core()->download_list()->erase(itr);
 
@@ -211,9 +177,7 @@ apply_remove_untied() {
 }
 
 torrent::Object
-apply_schedule(const torrent::Object& rawArgs) {
-  const torrent::Object::list_type& args = rawArgs.as_list();
-
+apply_schedule(const torrent::Object::list_type& args) {
   if (args.size() != 4)
     throw torrent::input_error("Wrong number of arguments.");
 
@@ -230,8 +194,7 @@ apply_schedule(const torrent::Object& rawArgs) {
 }
 
 torrent::Object
-apply_load(int flags, const torrent::Object& rawArgs) {
-  const torrent::Object::list_type&    args    = rawArgs.as_list();
+apply_load(const torrent::Object::list_type& args, int flags) {
   torrent::Object::list_const_iterator argsItr = args.begin();
 
   if (argsItr == args.end())
@@ -251,7 +214,7 @@ apply_load(int flags, const torrent::Object& rawArgs) {
 void apply_import(const std::string& path)     { if (!rpc::parse_command_file(path)) throw torrent::input_error("Could not open option file: " + path); }
 void apply_try_import(const std::string& path) { if (!rpc::parse_command_file(path)) control->core()->push_log_std("Could not read resource file: " + path); }
 
-void
+torrent::Object
 apply_close_low_diskspace(int64_t arg) {
   core::DownloadList* downloadList = control->core()->download_list();
 
@@ -274,11 +237,12 @@ apply_close_low_diskspace(int64_t arg) {
 
   if (closed)
     control->core()->push_log("Closed torrents due to low diskspace.");    
+
+  return torrent::Object();
 }
 
 torrent::Object
-apply_download_list(const torrent::Object& rawArgs) {
-  const torrent::Object::list_type&          args    = rawArgs.as_list();
+apply_download_list(const torrent::Object::list_type& args) {
   torrent::Object::list_const_iterator argsItr = args.begin();
 
   core::ViewManager* viewManager = control->view_manager();
@@ -296,7 +260,7 @@ apply_download_list(const torrent::Object& rawArgs) {
   torrent::Object::list_type& resultList = result.as_list();
 
   for (core::View::const_iterator itr = (*viewItr)->begin_visible(), last = (*viewItr)->end_visible(); itr != last; itr++) {
-    const torrent::HashString* hashString = &(*itr)->download()->info_hash();
+    const torrent::HashString* hashString = &(*itr)->info()->hash();
 
     resultList.push_back(rak::transform_hex(hashString->begin(), hashString->end()));
   }
@@ -305,23 +269,10 @@ apply_download_list(const torrent::Object& rawArgs) {
 }
 
 torrent::Object
-d_multicall(const torrent::Object& rawArgs) {
-  const torrent::Object::list_type&          args = rawArgs.as_list();
-
+d_multicall(const torrent::Object::list_type& args) {
   if (args.empty())
     throw torrent::input_error("Too few arguments.");
 
-//   const torrent::Object::string_type& infoHash = args.begin()->as_string();
-
-//   core::DownloadList*          dList = control->core()->download_list();
-//   core::DownloadList::iterator dItr  = dList->end();
-
-//   if (infoHash.size() == 40)
-//     dItr = dList->find_hex(infoHash.c_str());
-
-//   if (dItr == dList->end())
-//     throw torrent::input_error("Not a valid info-hash.");
-
   core::ViewManager* viewManager = control->view_manager();
   core::ViewManager::iterator viewItr;
 
@@ -350,48 +301,42 @@ d_multicall(const torrent::Object& rawArgs) {
   return resultRaw;
 }
 
+torrent::Object
+test_thread_locking() {
+  worker_thread->queue_item(&ThreadWorker::start_log_counter);
+
+  return torrent::Object();
+}
+
 void
 initialize_command_events() {
-  ADD_VARIABLE_BOOL("check_hash", true);
+  CMD2_ANY("test.thread_locking", std::tr1::bind(&test_thread_locking));
 
-  ADD_VARIABLE_BOOL("session_lock", true);
-  ADD_VARIABLE_BOOL("session_on_completion", true);
+  CMD2_ANY_STRING  ("on_ratio",        std::tr1::bind(&apply_on_ratio, std::tr1::placeholders::_2));
 
-  // Deprecated.
-  ADD_COMMAND_LIST("on_insert",       rak::bind_ptr_fn(&apply_on_state_change, "event.download.inserted"));
-  ADD_COMMAND_LIST("on_erase" ,       rak::bind_ptr_fn(&apply_on_state_change, "event.download.erased"));
-  ADD_COMMAND_LIST("on_open",         rak::bind_ptr_fn(&apply_on_state_change, "event.download.opened"));
-  ADD_COMMAND_LIST("on_close",        rak::bind_ptr_fn(&apply_on_state_change, "event.download.closed"));
-  ADD_COMMAND_LIST("on_start",        rak::bind_ptr_fn(&apply_on_state_change, "event.download.resumed"));
-  ADD_COMMAND_LIST("on_stop",         rak::bind_ptr_fn(&apply_on_state_change, "event.download.paused"));
-  ADD_COMMAND_LIST("on_hash_queued",  rak::bind_ptr_fn(&apply_on_state_change, "event.download.hash_queued"));
-  ADD_COMMAND_LIST("on_hash_removed", rak::bind_ptr_fn(&apply_on_state_change, "event.download.hash_removed"));
-  ADD_COMMAND_LIST("on_finished",     rak::bind_ptr_fn(&apply_on_state_change, "event.download.finished"));
+  CMD2_ANY         ("start_tied",      std::tr1::bind(&apply_start_tied));
+  CMD2_ANY         ("stop_untied",     std::tr1::bind(&apply_stop_untied));
+  CMD2_ANY         ("close_untied",    std::tr1::bind(&apply_close_untied));
+  CMD2_ANY         ("remove_untied",   std::tr1::bind(&apply_remove_untied));
 
-  ADD_COMMAND_STRING("on_ratio",      rak::ptr_fn(&apply_on_ratio));
+  CMD2_ANY_LIST    ("schedule2",        std::tr1::bind(&apply_schedule, std::tr1::placeholders::_2));
+  CMD2_ANY_STRING_V("schedule_remove2", std::tr1::bind(&rpc::CommandScheduler::erase_str, control->command_scheduler(), std::tr1::placeholders::_2));
 
-  ADD_COMMAND_VOID("start_tied",      &apply_start_tied);
-  ADD_COMMAND_VOID("stop_untied",     &apply_stop_untied);
-  ADD_COMMAND_VOID("close_untied",    &apply_close_untied);
-  ADD_COMMAND_VOID("remove_untied",   &apply_remove_untied);
+  CMD2_ANY_STRING_V("import",          std::tr1::bind(&apply_import, std::tr1::placeholders::_2));
+  CMD2_ANY_STRING_V("try_import",      std::tr1::bind(&apply_try_import, std::tr1::placeholders::_2));
 
-  ADD_COMMAND_LIST("schedule",                rak::ptr_fn(&apply_schedule));
-  ADD_COMMAND_STRING_UN("schedule_remove",    rak::make_mem_fun(control->command_scheduler(), &rpc::CommandScheduler::erase_str));
+  CMD2_ANY_LIST    ("load.normal",        std::tr1::bind(&apply_load, std::tr1::placeholders::_2, core::Manager::create_quiet | core::Manager::create_tied));
+  CMD2_ANY_LIST    ("load.verbose",       std::tr1::bind(&apply_load, std::tr1::placeholders::_2, core::Manager::create_tied));
+  CMD2_ANY_LIST    ("load.start",         std::tr1::bind(&apply_load, std::tr1::placeholders::_2,
+                                                         core::Manager::create_quiet | core::Manager::create_tied | core::Manager::create_start));
+  CMD2_ANY_LIST    ("load.start_verbose", std::tr1::bind(&apply_load, std::tr1::placeholders::_2, core::Manager::create_tied  | core::Manager::create_start));
+  CMD2_ANY_LIST    ("load.raw",           std::tr1::bind(&apply_load, std::tr1::placeholders::_2, core::Manager::create_quiet | core::Manager::create_raw_data));
+  CMD2_ANY_LIST    ("load.raw_verbose",   std::tr1::bind(&apply_load, std::tr1::placeholders::_2, core::Manager::create_raw_data));
+  CMD2_ANY_LIST    ("load.raw_start",     std::tr1::bind(&apply_load, std::tr1::placeholders::_2,
+                                                         core::Manager::create_quiet | core::Manager::create_start | core::Manager::create_raw_data));
 
-  ADD_COMMAND_STRING_UN("import",             std::ptr_fun(&apply_import));
-  ADD_COMMAND_STRING_UN("try_import",         std::ptr_fun(&apply_try_import));
+  CMD2_ANY_VALUE   ("close_low_diskspace", std::tr1::bind(&apply_close_low_diskspace, std::tr1::placeholders::_2));
 
-  ADD_COMMAND_LIST("load",                    rak::bind_ptr_fn(&apply_load, core::Manager::create_quiet | core::Manager::create_tied));
-  ADD_COMMAND_LIST("load_verbose",            rak::bind_ptr_fn(&apply_load, core::Manager::create_tied));
-  ADD_COMMAND_LIST("load_start",              rak::bind_ptr_fn(&apply_load, core::Manager::create_quiet | core::Manager::create_tied | core::Manager::create_start));
-  ADD_COMMAND_LIST("load_start_verbose",      rak::bind_ptr_fn(&apply_load, core::Manager::create_tied  | core::Manager::create_start));
-  ADD_COMMAND_LIST("load_raw",                rak::bind_ptr_fn(&apply_load, core::Manager::create_quiet | core::Manager::create_raw_data));
-  ADD_COMMAND_LIST("load_raw_verbose",        rak::bind_ptr_fn(&apply_load, core::Manager::create_raw_data));
-  ADD_COMMAND_LIST("load_raw_start",          rak::bind_ptr_fn(&apply_load, core::Manager::create_quiet | core::Manager::create_start | core::Manager::create_raw_data));
-
-  ADD_COMMAND_VALUE_UN("close_low_diskspace", std::ptr_fun(&apply_close_low_diskspace));
-
-  ADD_COMMAND_LIST("download_list",           rak::ptr_fn(&apply_download_list));
-  ADD_COMMAND_LIST("d.multicall",             rak::ptr_fn(&d_multicall));
-  ADD_COMMAND_COPY("call_download",           call_list, "i:", "");
+  CMD2_ANY_LIST    ("download_list",       std::tr1::bind(&apply_download_list, std::tr1::placeholders::_2));
+  CMD2_ANY_LIST    ("d.multicall2",        std::tr1::bind(&d_multicall, std::tr1::placeholders::_2));
 }
