@@ -36,6 +36,8 @@
 
 #include "config.h"
 
+#include <sys/types.h>
+
 #include <ctime>
 #include <rak/functional.h>
 #include <rak/functional_fun.h>
@@ -45,8 +47,6 @@
 #include "core/view_manager.h"
 #include "ui/root.h"
 #include "ui/download_list.h"
-#include "rpc/command_slot.h"
-#include "rpc/command_variable.h"
 #include "rpc/parse.h"
 
 #include "globals.h"
@@ -56,9 +56,7 @@
 typedef void (core::ViewManager::*view_cfilter_slot)(const std::string&, const std::string&);
 
 torrent::Object
-apply_view_filter_on(const torrent::Object& rawArgs) {
-  const torrent::Object::list_type& args = rawArgs.as_list();
-
+apply_view_filter_on(const torrent::Object::list_type& args) {
   if (args.size() < 1)
     throw torrent::input_error("Too few arguments.");
 
@@ -78,9 +76,7 @@ apply_view_filter_on(const torrent::Object& rawArgs) {
 }
 
 torrent::Object
-apply_view_cfilter(view_cfilter_slot viewFilterSlot, const torrent::Object& rawArgs) {
-  const torrent::Object::list_type& args = rawArgs.as_list();
-
+apply_view_cfilter(view_cfilter_slot viewFilterSlot, const torrent::Object::list_type& args) {
   if (args.size() != 2)
     throw torrent::input_error("Too few arguments.");
 
@@ -95,9 +91,7 @@ apply_view_cfilter(view_cfilter_slot viewFilterSlot, const torrent::Object& rawA
 }
 
 torrent::Object
-apply_view_sort(const torrent::Object& rawArgs) {
-  const torrent::Object::list_type& args = rawArgs.as_list();
-
+apply_view_sort(const torrent::Object::list_type& args) {
   if (args.size() <= 0 || args.size() > 2)
     throw torrent::input_error("Wrong argument count.");
 
@@ -117,7 +111,7 @@ apply_view_sort(const torrent::Object& rawArgs) {
 }
 
 torrent::Object
-apply_view_list(const torrent::Object&) {
+apply_view_list() {
   torrent::Object rawResult = torrent::Object::create_list();
   torrent::Object::list_type& result = rawResult.as_list();
 
@@ -128,9 +122,7 @@ apply_view_list(const torrent::Object&) {
 }
 
 torrent::Object
-apply_view_set(const torrent::Object& rawArgs) {
-  const torrent::Object::list_type& args = rawArgs.as_list();
-
+apply_view_set(const torrent::Object::list_type& args) {
   if (args.size() != 2)
     throw torrent::input_error("Wrong argument count.");
 
@@ -210,9 +202,7 @@ apply_or(rpc::target_type target, const torrent::Object& rawArgs) {
 }
 
 torrent::Object
-apply_less(rpc::target_type target, const torrent::Object& rawArgs) {
-  const torrent::Object::list_type& args = rawArgs.as_list();
-  
+apply_cmp(rpc::target_type target, const torrent::Object::list_type& args) {
   // We only need to check if empty() since if size() == 1 it calls
   // the same command for both, or if size() == 2 then each side of
   // the comparison has different commands.
@@ -237,20 +227,29 @@ apply_less(rpc::target_type target, const torrent::Object& rawArgs) {
     throw torrent::input_error("Type mismatch.");
     
   switch (result1.type()) {
-  case torrent::Object::TYPE_VALUE:  return result1.as_value() < result2.as_value();
-  case torrent::Object::TYPE_STRING: return result1.as_string() < result2.as_string();
-  default: return (int64_t)false;
+  case torrent::Object::TYPE_VALUE:  return result1.as_value() - result2.as_value();
+  case torrent::Object::TYPE_STRING: return result1.as_string().compare(result2.as_string());
+  default: return torrent::Object();
   }
 }
 
-// Fixme.
-torrent::Object
-apply_greater(rpc::target_type target, const torrent::Object& rawArgs) {
-  return (int64_t)!apply_less(target, rawArgs).as_value();
+torrent::Object apply_less(rpc::target_type target, const torrent::Object::list_type& args) {
+  torrent::Object result = apply_cmp(target, args);
+  return result.is_value() ? result.as_value() <  0 : (int64_t)false;
+}
+
+torrent::Object apply_greater(rpc::target_type target, const torrent::Object::list_type& args) {
+  torrent::Object result = apply_cmp(target, args);
+  return result.is_value() ? result.as_value() >  0 : (int64_t)false;
+}
+
+torrent::Object apply_equal(rpc::target_type target, const torrent::Object::list_type& args) {
+  torrent::Object result = apply_cmp(target, args);
+  return result.is_value() ? result.as_value() == 0 : (int64_t)false;
 }
 
 torrent::Object
-apply_to_time(int flags, const torrent::Object& rawArgs) {
+apply_to_time(const torrent::Object& rawArgs, int flags) {
   std::tm *u;
   time_t t = (uint64_t)rawArgs.as_value();
 
@@ -338,7 +337,7 @@ apply_to_throttle(const torrent::Object& rawArgs) {
 // if (cond1) { branch1 } else if (cond2) { branch2 } else { branch3 }
 // <cond1>,<branch1>,<cond2>,<branch2>,<branch3>
 torrent::Object
-apply_if(int flags, rpc::target_type target, const torrent::Object& rawArgs) {
+apply_if(rpc::target_type target, const torrent::Object& rawArgs, int flags) {
   const torrent::Object::list_type& args = rawArgs.as_list();
   torrent::Object::list_const_iterator itr = args.begin();
 
@@ -398,18 +397,17 @@ apply_if(int flags, rpc::target_type target, const torrent::Object& rawArgs) {
 }
 
 torrent::Object
-cmd_view_size(__UNUSED rpc::target_type target, const torrent::Object& rawArgs) {
-  return (*control->view_manager()->find_throw(rawArgs.as_string()))->size_visible();
+cmd_view_size(const torrent::Object::string_type& args) {
+  return (*control->view_manager()->find_throw(args))->size_visible();
 }
 
 torrent::Object
-cmd_view_size_not_visible(__UNUSED rpc::target_type target, const torrent::Object& rawArgs) {
-  return (*control->view_manager()->find_throw(rawArgs.as_string()))->size_not_visible();
+cmd_view_size_not_visible(const torrent::Object::string_type& args) {
+  return (*control->view_manager()->find_throw(args))->size_not_visible();
 }
 
 torrent::Object
-cmd_view_persistent(__UNUSED rpc::target_type target, const torrent::Object& rawArgs) {
-  const std::string& args = rawArgs.as_string();
+cmd_view_persistent(const torrent::Object::string_type& args) {
   core::View* view = *control->view_manager()->find_throw(args);
   
   if (!view->get_filter().empty() || !view->get_event_added().empty() || !view->get_event_removed().empty())
@@ -422,99 +420,98 @@ cmd_view_persistent(__UNUSED rpc::target_type target, const torrent::Object& raw
   return torrent::Object();
 }
 
+// TODO: These don't need wrapper functions anymore...
 torrent::Object
-cmd_ui_set_view(__UNUSED rpc::target_type target, const torrent::Object& rawArgs) {
-  control->ui()->download_list()->set_current_view(rawArgs.as_string());
-
+cmd_ui_set_view(const torrent::Object::string_type& args) {
+  control->ui()->download_list()->set_current_view(args);
   return torrent::Object();
 }
 
 torrent::Object
-cmd_ui_unfocus_download(core::Download* download, const torrent::Object& rawArgs) {
+cmd_ui_unfocus_download(core::Download* download) {
   control->ui()->download_list()->unfocus_download(download);
 
   return torrent::Object();
 }
 
 torrent::Object
-cmd_view_filter_download(core::Download* download, const torrent::Object& rawArgs) {
-  (*control->view_manager()->find_throw(rawArgs.as_string()))->filter_download(download);
+cmd_view_filter_download(core::Download* download, const torrent::Object::string_type& args) {
+  (*control->view_manager()->find_throw(args))->filter_download(download);
 
   return torrent::Object();
 }
 
 torrent::Object
-cmd_view_set_visible(core::Download* download, const torrent::Object& rawArgs) {
-  (*control->view_manager()->find_throw(rawArgs.as_string()))->set_visible(download);
+cmd_view_set_visible(core::Download* download, const torrent::Object::string_type& args) {
+  (*control->view_manager()->find_throw(args))->set_visible(download);
 
   return torrent::Object();
 }
 
 torrent::Object
-cmd_view_set_not_visible(core::Download* download, const torrent::Object& rawArgs) {
-  (*control->view_manager()->find_throw(rawArgs.as_string()))->set_not_visible(download);
+cmd_view_set_not_visible(core::Download* download, const torrent::Object::string_type& args) {
+  (*control->view_manager()->find_throw(args))->set_not_visible(download);
 
   return torrent::Object();
 }
 
 void
 initialize_command_ui() {
-  ADD_VARIABLE_STRING("key_layout", "qwerty");
+  CMD2_VAR_STRING("keys.layout", "qwerty");
 
-  ADD_COMMAND_STRING("view_add",        rpc::object_string_fn(rak::make_mem_fun(control->view_manager(), &core::ViewManager::insert_throw)));
-  ADD_COMMAND_NONE_L("view_list",       rak::ptr_fn(&apply_view_list));
-  ADD_COMMAND_NONE_L("view_set",        rak::ptr_fn(&apply_view_set));
+  CMD2_ANY_STRING("view.add", object_convert_void(std::tr1::bind(&core::ViewManager::insert_throw, control->view_manager(), std::tr1::placeholders::_2)));
 
-  ADD_COMMAND_LIST("view_filter",       rak::bind_ptr_fn(&apply_view_cfilter, &core::ViewManager::set_filter));
-  ADD_COMMAND_LIST("view_filter_on",    rak::ptr_fn(&apply_view_filter_on));
+  CMD2_ANY_L   ("view.list",          std::tr1::bind(&apply_view_list));
+  CMD2_ANY_LIST("view.set",           std::tr1::bind(&apply_view_set, std::tr1::placeholders::_2));
 
-  ADD_COMMAND_LIST("view_sort",         rak::ptr_fn(&apply_view_sort));
-  ADD_COMMAND_LIST("view_sort_new",     rak::bind_ptr_fn(&apply_view_cfilter, &core::ViewManager::set_sort_new));
-  ADD_COMMAND_LIST("view_sort_current", rak::bind_ptr_fn(&apply_view_cfilter, &core::ViewManager::set_sort_current));
+  CMD2_ANY_LIST("view.filter",        std::tr1::bind(&apply_view_cfilter, &core::ViewManager::set_filter, std::tr1::placeholders::_2));
+  CMD2_ANY_LIST("view.filter_on",     std::tr1::bind(&apply_view_filter_on, std::tr1::placeholders::_2));
 
-  ADD_COMMAND_LIST("view.event_added",   rak::bind_ptr_fn(&apply_view_cfilter, &core::ViewManager::set_event_added));
-  ADD_COMMAND_LIST("view.event_removed", rak::bind_ptr_fn(&apply_view_cfilter, &core::ViewManager::set_event_removed));
+  CMD2_ANY_LIST("view.sort",          std::tr1::bind(&apply_view_sort, std::tr1::placeholders::_2));
+  CMD2_ANY_LIST("view.sort_new",      std::tr1::bind(&apply_view_cfilter, &core::ViewManager::set_sort_new, std::tr1::placeholders::_2));
+  CMD2_ANY_LIST("view.sort_current",  std::tr1::bind(&apply_view_cfilter, &core::ViewManager::set_sort_current, std::tr1::placeholders::_2));
+
+  CMD2_ANY_LIST("view.event_added",   std::tr1::bind(&apply_view_cfilter, &core::ViewManager::set_event_added, std::tr1::placeholders::_2));
+  CMD2_ANY_LIST("view.event_removed", std::tr1::bind(&apply_view_cfilter, &core::ViewManager::set_event_removed, std::tr1::placeholders::_2));
 
   // Cleanup and add . to view.
 
-  CMD_G_STRING("view.size",              rak::ptr_fn(&cmd_view_size));
-  CMD_G_STRING("view.size_not_visible",  rak::ptr_fn(&cmd_view_size_not_visible));
-  CMD_G_STRING("view.persistent",        rak::ptr_fn(&cmd_view_persistent));
+  CMD2_ANY_STRING("view.size",              std::tr1::bind(&cmd_view_size, std::tr1::placeholders::_2));
+  CMD2_ANY_STRING("view.size_not_visible",  std::tr1::bind(&cmd_view_size_not_visible, std::tr1::placeholders::_2));
+  CMD2_ANY_STRING("view.persistent",        std::tr1::bind(&cmd_view_persistent, std::tr1::placeholders::_2));
 
-  CMD_D_STRING("view.filter_download",   rak::ptr_fn(&cmd_view_filter_download));
-  CMD_D_STRING("view.set_visible",       rak::ptr_fn(&cmd_view_set_visible));
-  CMD_D_STRING("view.set_not_visible",   rak::ptr_fn(&cmd_view_set_not_visible));
+  CMD2_DL_STRING ("view.filter_download", std::tr1::bind(&cmd_view_filter_download, std::tr1::placeholders::_1, std::tr1::placeholders::_2));
+  CMD2_DL_STRING ("view.set_visible",     std::tr1::bind(&cmd_view_set_visible,     std::tr1::placeholders::_1, std::tr1::placeholders::_2));
+  CMD2_DL_STRING ("view.set_not_visible", std::tr1::bind(&cmd_view_set_not_visible, std::tr1::placeholders::_1, std::tr1::placeholders::_2));
 
   // Commands that affect the default rtorrent UI.
-
-//   ADD_ANY_NONE("ui.focus",              rak::ptr_fn(&cmd_ui_focus));
-  CMD_D_ANY("ui.unfocus_download",      rak::ptr_fn(&cmd_ui_unfocus_download));
-  CMD_N_STRING("ui.current_view.set",   rak::ptr_fn(&cmd_ui_set_view));
+  CMD2_DL        ("ui.unfocus_download",   std::tr1::bind(&cmd_ui_unfocus_download, std::tr1::placeholders::_1));
+  CMD2_ANY_STRING("ui.current_view.set",   std::tr1::bind(&cmd_ui_set_view, std::tr1::placeholders::_2));
 
   // Move.
-
-  ADD_ANY_NONE("print",                 rak::ptr_fn(&apply_print));
-  ADD_ANY_NONE("cat",                   rak::ptr_fn(&apply_cat));
-  ADD_ANY_NONE("if",                    rak::bind_ptr_fn(&apply_if, 0));
-  ADD_ANY_NONE("not",                   rak::ptr_fn(&apply_not));
-  ADD_ANY_NONE("false",                 rak::ptr_fn(&apply_false));
-  ADD_ANY_NONE("and",                   rak::ptr_fn(&apply_and));
-  ADD_ANY_NONE("or",                    rak::ptr_fn(&apply_or));
-
-  ADD_ANY_LIST("less",                  rak::ptr_fn(&apply_less));
-  ADD_ANY_LIST("greater",               rak::ptr_fn(&apply_greater));
+  CMD2_ANY("print", &apply_print);
+  CMD2_ANY("cat",   &apply_cat);
+  CMD2_ANY("if",    std::tr1::bind(&apply_if, std::tr1::placeholders::_1, std::tr1::placeholders::_2, 0));
+  CMD2_ANY("not",   &apply_not);
+  CMD2_ANY("false", &apply_false);
+  CMD2_ANY("and",   &apply_and);
+  CMD2_ANY("or",    &apply_or);
 
   // A temporary command for handling stuff until we get proper
   // support for seperation of commands and literals.
-  ADD_ANY_NONE("branch",                rak::bind_ptr_fn(&apply_if, 1));
+  CMD2_ANY("branch", std::tr1::bind(&apply_if, std::tr1::placeholders::_1, std::tr1::placeholders::_2, 1));
 
-  ADD_COMMAND_VALUE("to_gm_time",       rak::bind_ptr_fn(&apply_to_time, 0));
-  ADD_COMMAND_VALUE("to_gm_date",       rak::bind_ptr_fn(&apply_to_time, 0x2));
-  ADD_COMMAND_VALUE("to_time",          rak::bind_ptr_fn(&apply_to_time, 0x1));
-  ADD_COMMAND_VALUE("to_date",          rak::bind_ptr_fn(&apply_to_time, 0x1 | 0x2));
-  ADD_COMMAND_VALUE("to_elapsed_time",  rak::ptr_fn(&apply_to_elapsed_time));
-  ADD_COMMAND_VALUE("to_kb",            rak::ptr_fn(&apply_to_kb));
-  ADD_COMMAND_VALUE("to_mb",            rak::ptr_fn(&apply_to_mb));
-  ADD_COMMAND_VALUE("to_xb",            rak::ptr_fn(&apply_to_xb));
-  ADD_COMMAND_VALUE("to_throttle",      rak::ptr_fn(&apply_to_throttle));
+  CMD2_ANY_LIST("less",    &apply_less);
+  CMD2_ANY_LIST("greater", &apply_greater);
+  CMD2_ANY_LIST("equal",   &apply_equal);
+
+  CMD2_ANY_VALUE("convert.gm_time",      std::tr1::bind(&apply_to_time, std::tr1::placeholders::_2, 0));
+  CMD2_ANY_VALUE("convert.gm_date",      std::tr1::bind(&apply_to_time, std::tr1::placeholders::_2, 0x2));
+  CMD2_ANY_VALUE("convert.time",         std::tr1::bind(&apply_to_time, std::tr1::placeholders::_2, 0x1));
+  CMD2_ANY_VALUE("convert.date",         std::tr1::bind(&apply_to_time, std::tr1::placeholders::_2, 0x1 | 0x2));
+  CMD2_ANY_VALUE("convert.elapsed_time", std::tr1::bind(&apply_to_elapsed_time, std::tr1::placeholders::_2));
+  CMD2_ANY_VALUE("convert.kb",           std::tr1::bind(&apply_to_kb, std::tr1::placeholders::_2));
+  CMD2_ANY_VALUE("convert.mb",           std::tr1::bind(&apply_to_mb, std::tr1::placeholders::_2));
+  CMD2_ANY_VALUE("convert.xb",           std::tr1::bind(&apply_to_xb, std::tr1::placeholders::_2));
+  CMD2_ANY_VALUE("convert.throttle",     std::tr1::bind(&apply_to_throttle, std::tr1::placeholders::_2));
 }
