@@ -53,24 +53,23 @@
 #include "rpc/command_scheduler.h"
 #include "rpc/parse_commands.h"
 #include "rpc/scgi.h"
+#include "rpc/object_storage.h"
 #include "ui/root.h"
 
 #include "control.h"
 
 Control::Control() :
-  m_shutdownReceived(false),
-  m_shutdownQuick(false),
-
   m_ui(new ui::Root()),
   m_display(new display::Manager()),
   m_input(new input::Manager()),
   m_inputStdin(new input::InputEvent(STDIN_FILENO)),
 
   m_commandScheduler(new rpc::CommandScheduler()),
+  m_objectStorage(new rpc::object_storage()),
 
-  m_scgi(NULL),
-
-  m_tick(0) {
+  m_tick(0),
+  m_shutdownReceived(false),
+  m_shutdownQuick(false) {
 
   m_core        = new core::Manager();
   m_viewManager = new core::ViewManager();
@@ -87,14 +86,15 @@ Control::~Control() {
   delete m_inputStdin;
   delete m_input;
 
-  delete m_commandScheduler;
-
   delete m_viewManager;
 
   delete m_ui;
   delete m_display;
   delete m_core;
   delete m_dhtManager;
+
+  delete m_commandScheduler;
+  delete m_objectStorage;
 }
 
 void
@@ -108,23 +108,23 @@ Control::initialize() {
 
   m_core->initialize_second();
   m_core->listen_open();
-  m_core->download_store()->enable(rpc::call_command_value("get_session_lock"));
+  m_core->download_store()->enable(rpc::call_command_value("session.use_lock"));
 
   m_core->set_hashing_view(*m_viewManager->find_throw("hashing"));
 
   m_ui->init(this);
 
-  m_inputStdin->insert(this_thread->poll());
+  m_inputStdin->insert(main_thread->poll());
 }
 
 void
 Control::cleanup() {
-  delete m_scgi; m_scgi = NULL;
+  //  delete m_scgi; m_scgi = NULL;
   rpc::xmlrpc.cleanup();
 
   priority_queue_erase(&taskScheduler, &m_taskShutdown);
 
-  m_inputStdin->remove(this_thread->poll());
+  m_inputStdin->remove(main_thread->poll());
 
   m_core->download_store()->disable();
 
@@ -139,14 +139,23 @@ Control::cleanup() {
 
 void
 Control::cleanup_exception() {
-  delete m_scgi; m_scgi = NULL;
+  //  delete m_scgi; m_scgi = NULL;
 
   display::Canvas::cleanup();
 }
 
+bool
+Control::is_shutdown_completed() {
+  return m_shutdownQuick && !worker_thread->is_active() && torrent::is_inactive();
+}
+
 void
 Control::handle_shutdown() {
   if (!m_shutdownQuick) {
+    // Temporary hack:
+    if (worker_thread->is_active())
+      worker_thread->queue_item(&ThreadBase::stop_thread);
+
     torrent::connection_manager()->listen_close();
     m_core->shutdown(false);
 
@@ -154,6 +163,10 @@ Control::handle_shutdown() {
       priority_queue_insert(&taskScheduler, &m_taskShutdown, cachedTime + rak::timer::from_seconds(5));
 
   } else {
+    // Temporary hack:
+    if (worker_thread->is_active())
+      worker_thread->queue_item(&ThreadBase::stop_thread);
+
     m_core->shutdown(true);
   }
 
