@@ -93,6 +93,7 @@ private:
 
   Manager*            m_manager;
   std::iostream*      m_stream;
+  torrent::Object*    m_object;
 
   bool                m_commited;
   bool                m_loaded;
@@ -112,6 +113,7 @@ private:
 };
 
 bool is_network_uri(const std::string& uri);
+bool is_magnet_uri(const std::string& uri);
 
 }
 
