@@ -39,6 +39,7 @@
 #include <cstdio>
 #include <cstring>
 #include <fstream>
+#include <sstream>
 #include <unistd.h>
 #include <sys/select.h>
 #include <rak/address_info.h>
@@ -52,6 +53,7 @@
 #include <torrent/connection_manager.h>
 #include <torrent/error.h>
 #include <torrent/exceptions.h>
+#include <torrent/object_stream.h>
 #include <torrent/resume.h>
 #include <torrent/tracker_list.h>
 #include <torrent/throttle.h>
@@ -77,7 +79,7 @@ namespace core {
 
 void
 receive_tracker_dump(const std::string& url, const char* data, size_t size) {
-  const std::string& filename = rpc::call_command_string("get_log.tracker");
+  const std::string& filename = rpc::call_command_string("log.tracker");
 
   if (filename.empty())
     return;
@@ -94,7 +96,7 @@ receive_tracker_dump(const std::string& url, const char* data, size_t size) {
 
 void
 Manager::handshake_log(const sockaddr* sa, int msg, int err, const torrent::HashString* hash) {
-  if (!rpc::call_command_value("get_handshake_log"))
+  if (!rpc::call_command_value("log.handshake"))
     return;
   
   std::string peer;
@@ -255,11 +257,11 @@ void
 Manager::listen_open() {
   // This stuff really should be moved outside of manager, make it
   // part of the init script.
-  if (!rpc::call_command_value("get_port_open"))
+  if (!rpc::call_command_value("network.port_open"))
     return;
 
   int portFirst, portLast;
-  torrent::Object portRange = rpc::call_command_void("get_port_range");
+  torrent::Object portRange = rpc::call_command_void("network.port_range");
 
   if (portRange.is_string()) {
     if (std::sscanf(portRange.as_string().c_str(), "%i-%i", &portFirst, &portLast) != 2)
@@ -274,7 +276,7 @@ Manager::listen_open() {
   if (portFirst > portLast || portLast >= (1 << 16))
     throw torrent::input_error("Invalid port range.");
 
-  if (rpc::call_command_value("get_port_random")) {
+  if (rpc::call_command_value("network.port_random")) {
     int boundary = portFirst + random() % (portLast - portFirst + 1);
 
     if (torrent::connection_manager()->listen_open(boundary, portLast) ||
@@ -395,6 +397,7 @@ Manager::try_create_download(const std::string& uri, int flags, const command_li
   if ((flags & create_tied) &&
       !(flags & create_raw_data) &&
       !is_network_uri(uri) &&
+      !is_magnet_uri(uri) &&
       !file_status_cache()->insert(uri, 0))
     return;
 
@@ -416,6 +419,31 @@ Manager::try_create_download(const std::string& uri, int flags, const command_li
   f->commit();
 }
 
+void
+Manager::try_create_download_from_meta_download(torrent::Object* bencode, const std::string& metafile) {
+  DownloadFactory* f = new DownloadFactory(this);
+
+  f->variables()["tied_to_file"] = (int64_t)true;
+  f->variables()["tied_file"] = metafile;
+
+  torrent::Object& meta = bencode->get_key("rtorrent_meta_download");
+  torrent::Object::list_type& commands = meta.get_key_list("commands");
+  for (torrent::Object::list_type::const_iterator itr = commands.begin(); itr != commands.end(); ++itr)
+    f->commands().insert(f->commands().end(), itr->as_string());
+
+  f->set_start(meta.get_key_value("start"));
+  f->set_print_log(meta.get_key_value("print_log"));
+  f->slot_finished(sigc::bind(sigc::ptr_fun(&rak::call_delete_func<core::DownloadFactory>), f));
+
+  // Bit of a waste to create the bencode repesentation here
+  // only to have the DownloadFactory decode it.
+  std::stringstream s;
+  s.imbue(std::locale::classic());
+  s << *bencode;
+  f->load_raw_data(s.str());
+  f->commit();
+}
+
 utils::Directory
 path_expand_transform(std::string path, const utils::directory_entry& entry) {
   return path + entry.d_name;
@@ -472,7 +500,7 @@ path_expand(std::vector<std::string>* paths, const std::string& pattern) {
 
 bool
 manager_equal_tied(const std::string& path, Download* download) {
-  return path == rpc::call_command_string("d.get_tied_to_file", rpc::make_target(download));
+  return path == rpc::call_command_string("d.tied_to_file", rpc::make_target(download));
 }
 
 void
@@ -513,7 +541,7 @@ Manager::receive_hashing_changed() {
       continue;
 
     bool tryQuick =
-      rpc::call_command_value("d.get_hashing", rpc::make_target(*itr)) == Download::variable_hashing_initial &&
+      rpc::call_command_value("d.hashing", rpc::make_target(*itr)) == Download::variable_hashing_initial &&
       (*itr)->download()->file_list()->bitfield()->empty();
 
     if (!tryQuick && foundHashing)
@@ -536,7 +564,7 @@ Manager::receive_hashing_changed() {
         (*itr)->download()->hash_stop();
 
         if (foundHashing) {
-          rpc::call_command_set_value("d.set_hashing", Download::variable_hashing_rehash, rpc::make_target(*itr));
+          rpc::call_command_set_value("d.hashing.set", Download::variable_hashing_rehash, rpc::make_target(*itr));
           continue;
         }
       }
@@ -547,7 +575,7 @@ Manager::receive_hashing_changed() {
     } catch (torrent::local_error& e) {
       if (tryQuick) {
         // Make sure we don't repeat the quick hashing.
-        rpc::call_command_set_value("d.set_hashing", Download::variable_hashing_rehash, rpc::make_target(*itr));
+        rpc::call_command_set_value("d.hashing.set", Download::variable_hashing_rehash, rpc::make_target(*itr));
 
       } else {
         (*itr)->set_hash_failed(true);
