@@ -85,11 +85,12 @@ struct view_downloads_filter : std::unary_function<Download*, bool> {
       torrent::Object result = rpc::parse_command_single(rpc::make_target(d1), m_command);
 
       switch (result.type()) {
-      case torrent::Object::TYPE_NONE:   return false;
-      case torrent::Object::TYPE_VALUE:  return result.as_value();
-      case torrent::Object::TYPE_STRING: return !result.as_string().empty();
-      case torrent::Object::TYPE_LIST:   return !result.as_list().empty();
-      case torrent::Object::TYPE_MAP:    return !result.as_map().empty();
+        //      case torrent::Object::TYPE_RAW_BENCODE: return !result.as_raw_bencode().empty();
+      case torrent::Object::TYPE_VALUE:       return result.as_value();
+      case torrent::Object::TYPE_STRING:      return !result.as_string().empty();
+      case torrent::Object::TYPE_LIST:        return !result.as_list().empty();
+      case torrent::Object::TYPE_MAP:         return !result.as_map().empty();
+      default: return false;
       }
 
       // The default filter action is to return true, to not filter
@@ -293,7 +294,7 @@ View::set_filter_on_event(const std::string& event) {
   if (std::find(m_events.begin(), m_events.end(), event) != m_events.end())
     return;
 
-  rpc::commands.call_catch("system.method.set_key", rpc::make_target(), rpc::create_object_list(event, "!view_" + m_name, "view.filter_download=" + m_name));
+  rpc::commands.call_catch("method.set_key", rpc::make_target(), rpc::create_object_list(event, "!view." + m_name, "view.filter_download=" + m_name));
   m_events.push_back(event);
 }
 
@@ -302,7 +303,7 @@ View::clear_filter_on() {
   // Don't clear insert and erase as these are required to keep the
   // View up-to-date with the available downloads.
   for (event_list_type::const_iterator itr = m_events.begin(); itr != m_events.end(); itr++)
-    rpc::commands.call_catch("system.method.set_key", rpc::make_target(), rpc::create_object_list(*itr, "!view_" + m_name));
+    rpc::commands.call_catch("method.set_key", rpc::make_target(), rpc::create_object_list(*itr, "!view." + m_name));
 }
 
 inline void
