@@ -62,8 +62,11 @@ ViewManager::clear() {
 
 ViewManager::iterator
 ViewManager::insert(const std::string& name) {
+  if (name.empty())
+    throw torrent::input_error("View with empty name not supported.");
+
   if (find(name) != end())
-    throw torrent::internal_error("ViewManager::insert(...) name already inserted.");
+    throw torrent::input_error("View with same name already inserted.");
 
   View* view = new View();
   view->initialize(name);
