@@ -50,6 +50,7 @@
 #include <torrent/tracker.h>
 #include <torrent/tracker_list.h>
 #include <torrent/data/file_list.h>
+#include <torrent/data/file_manager.h>
 #include <torrent/peer/client_info.h>
 
 #include "core/download.h"
@@ -127,14 +128,14 @@ print_address(char* first, char* last, const sockaddr* sa) {
 
 char*
 print_download_title(char* first, char* last, core::Download* d) {
-  return print_buffer(first, last, " %s", d->download()->name().c_str());
+  return print_buffer(first, last, " %s", d->info()->name().c_str());
 }
 
 char*
 print_download_info(char* first, char* last, core::Download* d) {
-  if (!d->download()->is_open())
+  if (!d->download()->info()->is_open())
     first = print_buffer(first, last, "[CLOSED]  ");
-  else if (!d->download()->is_active())
+  else if (!d->download()->info()->is_active())
     first = print_buffer(first, last, "[OPEN]    ");
   else
     first = print_buffer(first, last, "          ");
@@ -147,11 +148,11 @@ print_download_info(char* first, char* last, core::Download* d) {
                          (double)d->download()->file_list()->size_bytes() / (double)(1 << 20));
   
   first = print_buffer(first, last, " Rate: %5.1f / %5.1f KB Uploaded: %7.1f MB",
-                       (double)d->download()->up_rate()->rate() / (1 << 10),
-                       (double)d->download()->down_rate()->rate() / (1 << 10),
-                       (double)d->download()->up_rate()->total() / (1 << 20));
+                       (double)d->info()->up_rate()->rate() / (1 << 10),
+                       (double)d->info()->down_rate()->rate() / (1 << 10),
+                       (double)d->info()->up_rate()->total() / (1 << 20));
 
-  if (d->download()->is_active() && !d->is_done()) {
+  if (d->download()->info()->is_active() && !d->is_done()) {
     first = print_buffer(first, last, " ");
     first = print_download_percentage_done(first, last, d);
 
@@ -162,15 +163,15 @@ print_download_info(char* first, char* last, core::Download* d) {
   }
 
   first = print_buffer(first, last, " [%c%c R: %4.2f",
-                       rpc::call_command_string("d.get_tied_to_file", rpc::make_target(d)).empty() ? ' ' : 'T',
-                       rpc::call_command_value("d.get_ignore_commands", rpc::make_target(d)) == 0 ? ' ' : 'I',
-                       (double)rpc::call_command_value("d.get_ratio", rpc::make_target(d)) / 1000.0);
+                       rpc::call_command_string("d.tied_to_file", rpc::make_target(d)).empty() ? ' ' : 'T',
+                       rpc::call_command_value("d.ignore_commands", rpc::make_target(d)) == 0 ? ' ' : 'I',
+                       (double)rpc::call_command_value("d.ratio", rpc::make_target(d)) / 1000.0);
 
   if (d->priority() != 2)
-    first = print_buffer(first, last, " %s", rpc::call_command_string("d.get_priority_str", rpc::make_target(d)).c_str());
+    first = print_buffer(first, last, " %s", rpc::call_command_string("d.priority_str", rpc::make_target(d)).c_str());
 
   if (!d->bencode()->get_key("rtorrent").get_key_string("throttle_name").empty())
-    first = print_buffer(first, last , " %s", rpc::call_command_string("d.get_throttle_name", rpc::make_target(d)).c_str());
+    first = print_buffer(first, last , " %s", rpc::call_command_string("d.throttle_name", rpc::make_target(d)).c_str());
 
   first = print_buffer(first, last , "]");
 
@@ -184,7 +185,7 @@ char*
 print_download_status(char* first, char* last, core::Download* d) {
   if (d->is_active())
     ;
-  else if (rpc::call_command_value("d.get_hashing", rpc::make_target(d)) != 0)
+  else if (rpc::call_command_value("d.hashing", rpc::make_target(d)) != 0)
     first = print_buffer(first, last, "Hashing: ");
   else if (!d->is_active())
     first = print_buffer(first, last, "Inactive: ");
@@ -216,7 +217,7 @@ print_download_status(char* first, char* last, core::Download* d) {
 
 char*
 print_download_time_left(char* first, char* last, core::Download* d) {
-  uint32_t rate = d->download()->down_rate()->rate();
+  uint32_t rate = d->info()->down_rate()->rate();
 
   if (rate < 512)
     return print_buffer(first, last, "--d --:--");
@@ -309,8 +310,8 @@ print_status_extra(char* first, char* last) {
                        torrent::connection_manager()->max_size());
                        
   first = print_buffer(first, last, " [F %i/%i]",
-                       torrent::open_files(),
-                       torrent::max_open_files());
+                       torrent::file_manager()->open_files(),
+                       torrent::file_manager()->max_open_files());
 
   return first;
 }
