@@ -57,27 +57,17 @@ struct command_map_data_type {
   //
   // The any_slot should perhaps replace generic_slot?
 
-  command_map_data_type(Command* variable, int flags, const char* parm, const char* doc) :
-    m_variable(variable), m_flags(flags), m_parm(parm), m_doc(doc) {}
+  command_map_data_type(int flags, const char* parm, const char* doc) :
+    m_flags(flags), m_parm(parm), m_doc(doc) {}
 
-  int                 target() const { return m_target; }
-
-  Command*            m_variable;
-
-  union {
-    Command::cleaned_slot  m_genericSlot;
-    Command::any_slot      m_anySlot;
-    Command::download_slot m_downloadSlot;
-    Command::file_slot     m_fileSlot;
-    Command::file_itr_slot m_fileItrSlot;
-    Command::peer_slot     m_peerSlot;
-    Command::tracker_slot  m_trackerSlot;
-
-    Command::download_pair_slot m_downloadPairSlot;
-  };
+  command_map_data_type(const command_map_data_type& src) :
+    m_variable(src.m_variable), m_anySlot(src.m_anySlot),
+    m_flags(src.m_flags), m_parm(src.m_parm), m_doc(src.m_doc) {}
+  
+  command_base             m_variable;
+  command_base::any_slot   m_anySlot;
 
   int           m_flags;
-  int           m_target;
 
   const char*   m_parm;
   const char*   m_doc;
@@ -102,8 +92,13 @@ public:
   static const int flag_dont_delete   = 0x1;
   static const int flag_delete_key    = 0x2;
   static const int flag_public_xmlrpc = 0x4;
-  static const int flag_no_target     = 0x8;
   static const int flag_modifiable    = 0x10;
+  static const int flag_is_redirect   = 0x20;
+  static const int flag_has_redirects = 0x40;
+
+  static const int flag_no_target      = 0x100;
+  static const int flag_file_target    = 0x200;
+  static const int flag_tracker_target = 0x400;
 
   CommandMap() {}
   ~CommandMap();
@@ -113,38 +108,39 @@ public:
 
   bool                is_modifiable(const_iterator itr) { return itr != end() && (itr->second.m_flags & flag_modifiable); }
 
-  iterator            insert(key_type key, Command* variable, int flags, const char* parm, const char* doc);
+  iterator            insert(key_type key, int flags, const char* parm, const char* doc);
 
-  // Make this a wrapper call to insert without extra fluff.
-  template <typename T>
-  void                insert_type(key_type key, Command* variable, T targetSlot, int flags, const char* parm, const char* doc) {
-    iterator itr = insert(key, variable, flags, parm, doc);
-
-    itr->second.m_target      = target_type_id<T>::value; 
-    itr->second.m_genericSlot = (Command::cleaned_slot)targetSlot;
+  template <typename T, typename Slot>
+  void
+  insert_slot(key_type key, Slot variable, command_base::any_slot targetSlot, int flags, const char* parm, const char* doc) {
+    iterator itr = insert(key, flags, parm, doc);
+    itr->second.m_variable.set_function<T>(variable);
+    itr->second.m_anySlot = targetSlot;
   }
 
-  void                insert(key_type key, const command_map_data_type src);
+  //  void                insert(key_type key, const command_map_data_type src);
   void                erase(iterator itr);
 
+  void                create_redirect(key_type key_new, key_type key_dest, int flags);
+
   const mapped_type   call(key_type key, const mapped_type& args = mapped_type());
   const mapped_type   call(key_type key, target_type target, const mapped_type& args = mapped_type()) { return call_command(key, args, target); }
   const mapped_type   call_catch(key_type key, target_type target, const mapped_type& args = mapped_type(), const char* err = "Command failed: ");
 
-  const mapped_type   call_command  (key_type key,       const mapped_type& arg, target_type target = target_type((int)Command::target_generic, NULL));
-  const mapped_type   call_command  (const_iterator itr, const mapped_type& arg, target_type target = target_type((int)Command::target_generic, NULL));
+  const mapped_type   call_command  (key_type key, const mapped_type& arg, target_type target = target_type((int)command_base::target_generic, NULL));
+  const mapped_type   call_command  (iterator itr, const mapped_type& arg, target_type target = target_type((int)command_base::target_generic, NULL));
 
-  const mapped_type   call_command_d(key_type key, core::Download* download, const mapped_type& arg)  { return call_command(key, arg, target_type((int)Command::target_download, download)); }
-  const mapped_type   call_command_p(key_type key, torrent::Peer* peer, const mapped_type& arg)       { return call_command(key, arg, target_type((int)Command::target_peer, peer)); }
-  const mapped_type   call_command_t(key_type key, torrent::Tracker* tracker, const mapped_type& arg) { return call_command(key, arg, target_type((int)Command::target_tracker, tracker)); }
-  const mapped_type   call_command_f(key_type key, torrent::File* file, const mapped_type& arg)       { return call_command(key, arg, target_type((int)Command::target_file, file)); }
+  const mapped_type   call_command_d(key_type key, core::Download* download, const mapped_type& arg)  { return call_command(key, arg, target_type((int)command_base::target_download, download)); }
+  const mapped_type   call_command_p(key_type key, torrent::Peer* peer, const mapped_type& arg)       { return call_command(key, arg, target_type((int)command_base::target_peer, peer)); }
+  const mapped_type   call_command_t(key_type key, torrent::Tracker* tracker, const mapped_type& arg) { return call_command(key, arg, target_type((int)command_base::target_tracker, tracker)); }
+  const mapped_type   call_command_f(key_type key, torrent::File* file, const mapped_type& arg)       { return call_command(key, arg, target_type((int)command_base::target_file, file)); }
 
 private:
   CommandMap(const CommandMap&);
   void operator = (const CommandMap&);
 };
 
-inline target_type make_target()                                  { return target_type((int)Command::target_generic, NULL); }
+inline target_type make_target()                                  { return target_type((int)command_base::target_generic, NULL); }
 inline target_type make_target(int type, void* target)            { return target_type(type, target); }
 inline target_type make_target(int type, void* target1, void* target2) { return target_type(type, target1, target2); }
 
