@@ -49,6 +49,7 @@
 #include "control.h"
 #include "globals.h"
 #include "scgi.h"
+#include "parse_commands.h"
 
 namespace rpc {
 
@@ -118,16 +119,16 @@ SCgi::open(void* sa, unsigned int length) {
 
 void
 SCgi::activate() {
-  this_thread->poll()->open(this);
-  this_thread->poll()->insert_read(this);
-  this_thread->poll()->insert_error(this);
+  worker_thread->poll()->open(this);
+  worker_thread->poll()->insert_read(this);
+  worker_thread->poll()->insert_error(this);
 }
 
 void
 SCgi::deactivate() {
-  this_thread->poll()->remove_read(this);
-  this_thread->poll()->remove_error(this);
-  this_thread->poll()->close(this);
+  worker_thread->poll()->remove_read(this);
+  worker_thread->poll()->remove_error(this);
+  worker_thread->poll()->close(this);
 }
 
 void
@@ -163,7 +164,14 @@ SCgi::receive_call(SCgiTask* task, const char* buffer, uint32_t length) {
   slot_write slotWrite;
   slotWrite.set(rak::mem_fn(task, &SCgiTask::receive_write));
 
-  return m_slotProcess(buffer, length, slotWrite);
+  ThreadBase::acquire_global_lock();
+  ThreadBase::interrupt_main_polling();
+
+  bool result = xmlrpc.process(buffer, length, slotWrite);
+
+  ThreadBase::release_global_lock();
+
+  return result;
 }
 
 }
