@@ -41,6 +41,8 @@
 #include <xmlrpc-c/server.h>
 #endif
 
+#include <cctype>
+#include <rak/string_manip.h>
 #include <torrent/object.h>
 #include <torrent/exceptions.h>
 
@@ -175,29 +177,38 @@ xmlrpc_to_target(xmlrpc_env* env, xmlrpc_value* value) {
     // Trackers: "<hash>:t<index>"
 
     int index;
-    const char* end;
+    const char* end_ptr = str + 42;
 
     switch (str[41]) {
     case 'f':
-      end = str + 42;
-      index = ::strtol(str + 42, (char**)&end, 0);
+      index = ::strtol(str + 42, (char**)&end_ptr, 0);
 
-      if (*str == '\0' || *end != '\0')
+      if (*str == '\0' || *end_ptr != '\0')
         throw xmlrpc_error(XMLRPC_TYPE_ERROR, "Invalid index.");
 
       target = rpc::make_target(XmlRpc::call_file, xmlrpc.get_slot_find_file()(download, index));
       break;
 
     case 't':
-      end = str + 42;
-      index = ::strtol(str + 42, (char**)&end, 0);
+      index = ::strtol(str + 42, (char**)&end_ptr, 0);
 
-      if (*str == '\0' || *end != '\0')
+      if (*str == '\0' || *end_ptr != '\0')
         throw xmlrpc_error(XMLRPC_TYPE_ERROR, "Invalid index.");
 
       target = rpc::make_target(XmlRpc::call_tracker, xmlrpc.get_slot_find_tracker()(download, index));
       break;
 
+    case 'p':
+    {
+      torrent::HashString hash;
+      const char* hash_end = torrent::hash_string_from_hex_c_str(str + 42, hash);
+
+      if (hash_end == end_ptr || *hash_end != '\0')
+        throw xmlrpc_error(XMLRPC_TYPE_ERROR, "Not a hash string.");
+
+      target = rpc::make_target(XmlRpc::call_peer, xmlrpc.get_slot_find_peer()(download, hash));
+      break;
+    }
     default:
       ::free((void*)str);
       throw xmlrpc_error(XMLRPC_TYPE_ERROR, "Unsupported target type found.");
@@ -303,7 +314,7 @@ xmlrpc_to_object(xmlrpc_env* env, xmlrpc_value* value, int callType, rpc::target
     if (env->fault_occurred)
       throw xmlrpc_error(env);
 
-    if (callType != XmlRpc::call_generic) {
+    if (callType != XmlRpc::call_generic && last != 0) {
       if (last < 1)
         throw xmlrpc_error(XMLRPC_TYPE_ERROR, "Too few arguments.");
 
@@ -326,7 +337,7 @@ xmlrpc_to_object(xmlrpc_env* env, xmlrpc_value* value, int callType, rpc::target
         // parameter as the index to support old-style calls.
 
         if (current == last)
-          throw xmlrpc_error(XMLRPC_TYPE_ERROR, "Too few arguments.");
+          throw xmlrpc_error(XMLRPC_TYPE_ERROR, "Too few arguments, missing index.");
 
         *target = xmlrpc_to_index_type(xmlrpc_list_entry_to_value(env, value, current++), callType, (core::Download*)target->second);
       }
@@ -432,7 +443,7 @@ object_to_xmlrpc(xmlrpc_env* env, const torrent::Object& object) {
 
 xmlrpc_value*
 xmlrpc_call_command(xmlrpc_env* env, xmlrpc_value* args, void* voidServerInfo) {
-  CommandMap::const_iterator itr = commands.find((const char*)voidServerInfo);
+  CommandMap::iterator itr = commands.find((const char*)voidServerInfo);
 
   if (itr == commands.end()) {
     xmlrpc_env_set_fault(env, XMLRPC_PARSE_ERROR, ("Command \"" + std::string((const char*)voidServerInfo) + "\" does not exist.").c_str());
@@ -445,8 +456,12 @@ xmlrpc_call_command(xmlrpc_env* env, xmlrpc_value* args, void* voidServerInfo) {
 
     if (itr->second.m_flags & CommandMap::flag_no_target)
       xmlrpc_to_object(env, args, XmlRpc::call_generic, &target).swap(object);
+    else if (itr->second.m_flags & CommandMap::flag_file_target)
+      xmlrpc_to_object(env, args, XmlRpc::call_file, &target).swap(object);
+    else if (itr->second.m_flags & CommandMap::flag_tracker_target)
+      xmlrpc_to_object(env, args, XmlRpc::call_tracker, &target).swap(object);
     else
-      xmlrpc_to_object(env, args, itr->second.target(), &target).swap(object);
+      xmlrpc_to_object(env, args, XmlRpc::call_any, &target).swap(object);
 
     if (env->fault_occurred)
       return NULL;
@@ -492,6 +507,9 @@ XmlRpc::process(const char* inBuffer, uint32_t length, slot_write slotWrite) {
 
   xmlrpc_mem_block* memblock = xmlrpc_registry_process_call(&localEnv, (xmlrpc_registry*)m_registry, NULL, inBuffer, length);
 
+  if (localEnv.fault_occurred && localEnv.fault_code == XMLRPC_INTERNAL_ERROR)
+    throw torrent::internal_error("Internal error in XMLRPC.");
+
   bool result = slotWrite((const char*)xmlrpc_mem_block_contents(memblock),
                           xmlrpc_mem_block_size(memblock));
 
