@@ -149,52 +149,52 @@ Download::create_info() {
 
   // Get these bindings with some kind of string map.
 
-  element->push_column("Name:",             te_command("d.get_name="));
-  element->push_column("Local id:",         te_command("d.get_local_id_html="));
-  element->push_column("Info hash:",        te_command("d.get_hash="));
-  element->push_column("Created:",          te_command("cat=$to_date=$d.get_creation_date=,\" \",$to_time=$d.get_creation_date="));
+  element->push_column("Name:",             te_command("d.name="));
+  element->push_column("Local id:",         te_command("d.local_id_html="));
+  element->push_column("Info hash:",        te_command("d.hash="));
+  element->push_column("Created:",          te_command("cat=$convert.date=$d.creation_date=,\" \",$convert.time=$d.creation_date="));
 
   element->push_back("");
-  element->push_column("Directory:",        te_command("d.get_directory="));
-  element->push_column("Base Path:",        te_command("d.get_base_path="));
-  element->push_column("Tied to file:",     te_command("d.get_tied_to_file="));
-  element->push_column("File stats:",       te_command("cat=$if=$d.is_multi_file=\\,multi\\,single,\" \",$d.get_size_files=,\" files\""));
+  element->push_column("Directory:",        te_command("d.directory="));
+  element->push_column("Base Path:",        te_command("d.base_path="));
+  element->push_column("Tied to file:",     te_command("d.tied_to_file="));
+  element->push_column("File stats:",       te_command("cat=$if=$d.is_multi_file=\\,multi\\,single,\" \",$d.size_files=,\" files\""));
 
   element->push_back("");
-  element->push_column("Chunks:",           te_command("cat=$d.get_completed_chunks=,\" / \",$d.get_size_chunks=,\" * \",$d.get_chunk_size="));
-  element->push_column("Priority:",         te_command("d.get_priority="));
-  element->push_column("Peer exchange:",    te_command("cat=$if=$d.get_peer_exchange=\\,enabled\\,disabled,\\ ,"
+  element->push_column("Chunks:",           te_command("cat=$d.completed_chunks=,\" / \",$d.size_chunks=,\" * \",$d.chunk_size="));
+  element->push_column("Priority:",         te_command("d.priority="));
+  element->push_column("Peer exchange:",    te_command("cat=$if=$d.peer_exchange=\\,enabled\\,disabled,\\ ,"
                                                        "$if=$d.is_pex_active=\\,active\\,$d.is_private=\\,private\\,inactive,"
-                                                       "\\ (,$d.get_size_pex=,/,$d.get_max_size_pex=,)"));
+                                                       "\\ (,$d.size_pex=,/,$d.max_size_pex=,)"));
 
-  element->push_column("State changed:",    te_command("to_elapsed_time=$d.get_state_changed="));
+  element->push_column("State changed:",    te_command("convert.elapsed_time=$d.state_changed="));
 
   element->push_back("");
-  element->push_column("Memory usage:",     te_command("cat=$to_mb=$get_memory_usage=,\" MB\""));
-  element->push_column("Max memory usage:", te_command("cat=$to_mb=$get_max_memory_usage=,\" MB\""));
-  element->push_column("Free diskspace:",   te_command("cat=$to_mb=$d.get_free_diskspace=,\" MB\""));
-  element->push_column("Safe diskspace:",   te_command("cat=$to_mb=$get_safe_free_diskspace=,\" MB\""));
+  element->push_column("Memory usage:",     te_command("cat=$convert.mb=$pieces.memory.current=,\" MB\""));
+  element->push_column("Max memory usage:", te_command("cat=$convert.mb=$pieces.memory.max=,\" MB\""));
+  element->push_column("Free diskspace:",   te_command("cat=$convert.mb=$d.free_diskspace=,\" MB\""));
+  element->push_column("Safe diskspace:",   te_command("cat=$convert.mb=$pieces.sync.safe_free_diskspace=,\" MB\""));
 
   element->push_back("");
-  element->push_column("Connection type:",  te_command("d.get_connection_current="));
-  element->push_column("Safe sync:",        te_command("if=$get_safe_sync=,yes,no"));
-  element->push_column("Send buffer:",      te_command("cat=$to_kb=$get_send_buffer_size=,\" KB\""));
-  element->push_column("Receive buffer:",   te_command("cat=$to_kb=$get_receive_buffer_size=,\" KB\""));
+  element->push_column("Connection type:",  te_command("d.connection_current="));
+  element->push_column("Safe sync:",        te_command("if=$pieces.sync.always_safe=,yes,no"));
+  element->push_column("Send buffer:",      te_command("cat=$convert.kb=$network.send_buffer.size=,\" KB\""));
+  element->push_column("Receive buffer:",   te_command("cat=$convert.kb=$network.receive_buffer.size=,\" KB\""));
 
   // TODO: Define a custom command for this and use $argument.0 instead of looking up the name multiple times?
-  element->push_column("Throttle:",         te_command("branch=d.get_throttle_name=,\""
-                                                              "cat=$d.get_throttle_name=,\\\"  [Max \\\","
-                                                                  "$to_throttle=$get_throttle_up_max=$d.get_throttle_name=,\\\"/\\\","
-                                                                  "$to_throttle=$get_throttle_down_max=$d.get_throttle_name=,\\\" KB]  [Rate \\\","
-                                                                  "$to_kb=$get_throttle_up_rate=$d.get_throttle_name=,\\\"/\\\","
-                                                                  "$to_kb=$get_throttle_down_rate=$d.get_throttle_name=,\\\" KB]\\\"\","
+  element->push_column("Throttle:",         te_command("branch=d.throttle_name=,\""
+                                                              "cat=$d.throttle_name=,\\\"  [Max \\\","
+                                                                  "$convert.throttle=$throttle.up.max=$d.throttle_name=,\\\"/\\\","
+                                                                  "$convert.throttle=$throttle.down.max=$d.throttle_name=,\\\" KB]  [Rate \\\","
+                                                                  "$convert.kb=$throttle.up.rate=$d.throttle_name=,\\\"/\\\","
+                                                                  "$convert.kb=$throttle.down.rate=$d.throttle_name=,\\\" KB]\\\"\","
                                                               "cat=\"global\""));
 
   element->push_back("");
-  element->push_column("Upload:",           te_command("cat=$to_kb=$d.get_up_rate=,\" KB / \",$to_xb=$d.get_up_total="));
-  element->push_column("Download:",         te_command("cat=$to_kb=$d.get_down_rate=,\" KB / \",$to_xb=$d.get_down_total="));
-  element->push_column("Skipped:",          te_command("cat=$to_kb=$d.get_skip_rate=,\" KB / \",$to_xb=$d.get_skip_total="));
-  element->push_column("Preload:",          te_command("cat=$get_preload_type=,\" / \",$get_stats_preloaded=,\" / \",$get_stats_not_preloaded="));
+  element->push_column("Upload:",           te_command("cat=$convert.kb=$d.up.rate=,\" KB / \",$convert.xb=$d.up.total="));
+  element->push_column("Download:",         te_command("cat=$convert.kb=$d.down.rate=,\" KB / \",$convert.xb=$d.down.total="));
+  element->push_column("Skipped:",          te_command("cat=$convert.kb=$d.skip.rate=,\" KB / \",$convert.xb=$d.skip.total="));
+  element->push_column("Preload:",          te_command("cat=$pieces.preload.type=,\" / \",$pieces.stats_preloaded=,\" / \",$pieces.stats_preloaded="));
 
   element->set_column_width(element->column_width() + 1);
 
@@ -289,7 +289,7 @@ Download::activate_display(Display displayType, bool focusDisplay) {
   // Set title.
   switch (displayType) {
   case DISPLAY_MAX_SIZE: break;
-  default: control->ui()->window_title()->set_title(m_download->download()->name()); break;
+  default: control->ui()->window_title()->set_title(m_download->info()->name()); break;
   }
 
   control->display()->adjust_layout();
@@ -299,21 +299,21 @@ void
 Download::receive_max_uploads(int t) {
   m_windowDownloadStatus->mark_dirty();
 
-  m_download->download()->set_uploads_max(std::max(m_download->download()->uploads_max() + t, (uint32_t)2));
+  m_download->download()->set_uploads_max(std::max<int32_t>(m_download->download()->uploads_max() + t, 0));
 }
 
 void
 Download::receive_min_peers(int t) {
   m_windowDownloadStatus->mark_dirty();
 
-  m_download->download()->connection_list()->set_min_size(std::max(m_download->download()->connection_list()->min_size() + t, (uint32_t)5));
+  m_download->download()->connection_list()->set_min_size(std::max<int32_t>(m_download->download()->connection_list()->min_size() + t, (int32_t)5));
 }
 
 void
 Download::receive_max_peers(int t) {
   m_windowDownloadStatus->mark_dirty();
 
-  m_download->download()->connection_list()->set_max_size(std::max(m_download->download()->connection_list()->max_size() + t, (uint32_t)5));
+  m_download->download()->connection_list()->set_max_size(std::max<int32_t>(m_download->download()->connection_list()->max_size() + t, (int32_t)5));
 }
 
 void
