@@ -220,7 +220,9 @@ DownloadList::activate_display(Display displayType) {
   // Set title.
   switch (displayType) {
   case DISPLAY_DOWNLOAD_LIST:
-    control->ui()->window_title()->set_title("rTorrent " VERSION "/" + std::string(torrent::version()) + " - " + rpc::call_command_string("get_name"));
+    control->ui()->window_title()->set_title("rTorrent " VERSION "/" +
+                                             std::string(torrent::version()) + " - " +
+                                             rpc::call_command_string("session.name"));
     break;
   case DISPLAY_LOG:
     control->ui()->window_title()->set_title("Log");
@@ -242,17 +244,17 @@ DownloadList::receive_view_input(Input type) {
 
   switch (type) {
   case INPUT_LOAD_DEFAULT:
-    title = "load_start";
+    title = "load.start";
     break;
 
   case INPUT_LOAD_MODIFIED:
-    title = "load";
+    title = "load.normal";
     break;
 
   case INPUT_CHANGE_DIRECTORY:
     title = "change_directory";
 
-    input->str() = rpc::call_command_string("get_directory");
+    input->str() = rpc::call_command_string("directory.default");
     input->set_pos(input->str().length());
 
     break;
@@ -311,8 +313,8 @@ DownloadList::receive_exit_input(Input type) {
       if ((*current_view()->focus())->is_open())
         throw torrent::input_error("Cannot change root directory on an open download.");
 
-      rpc::call_command("d.set_directory", rak::trim(input->str()), rpc::make_target(*current_view()->focus()));
-      control->core()->push_log_std("New root directory \"" + rpc::call_command_string("d.get_directory", rpc::make_target(*current_view()->focus())) + "\" for torrent.");
+      rpc::call_command("d.directory.set", rak::trim(input->str()), rpc::make_target(*current_view()->focus()));
+      control->core()->push_log_std("New root directory \"" + rpc::call_command_string("d.directory", rpc::make_target(*current_view()->focus())) + "\" for torrent.");
       break;
 
     case INPUT_COMMAND:
